
local addon = CreateFrame( "Frame", "hcRoleBotFrame" )

addon.state = {
	role = nil, -- Cached role.
}

--- Dispatches events to this addon by calling the method when the 
-- same name as the given event.
function addon:OnEvent( event, ... )
	if self[ event ] then
		self[ event ]( self, event, ... )
	end
end

--- Fired when the login process is complete.
-- This registers the events fired when critters should be summoned 
-- or dismissed.
function addon:PLAYER_LOGIN( event )
	if not IsLoggedIn( ) then
		return
	end
	
	self:UnregisterAllEvents( )
	
	self:RegisterEvent( "ACTIVE_TALENT_GROUP_CHANGED" )
	self:RegisterEvent( "PARTY_MEMBERS_CHANGED" )
	self:RegisterEvent( "ROLE_CHANGED_INFORM" )
	self:RegisterEvent( "ROLE_POLL_BEGIN" )
end

-- @return true if the player is in a party or raid group.
local function _IsPlayerInGroup( )
	if GetNumPartyMembers( ) > 0 then
		return true
	end
	
	if GetNumRaidMembers( ) > 0 then
		return true
	end
	
	return false
end

-- Checks the Feral talent tree for the presence of tanking talents.
-- @return "TANK" if tanking talents were found, "DAMAGER" otherwise.
local function _GetFeralDruidRole( )
	-- Check for "Thick Hide"
	local _, _, _, _, rank, _, _, _, _, _ = GetTalentInfo( 2, 11, false, false )
	if rank > 0 then
		-- Probably tank.
		return "TANK"
	end
	
	-- Check for "Predatory Strikes"
	local _, _, _, _, rank, _, _, _, _, _ = GetTalentInfo( 2, 3, false, false )
	if rank > 0 then
		-- Probably damage.
		return "DAMAGER"
	end
	
	-- Likely too low-level to make a meaningful distinction.
	return "NONE"
end

-- Defines the roles for particular classes and specs.
local _ROLES = {
	[ "DEATHKNIGHT" ] = { "TANK", "DAMAGER", "DAMAGER", },
	[ "DRUID" ] = { "DAMAGER", _GetFeralDruidRole( ), "HEALER", },
	[ "HUNTER" ] = "DAMAGER",
	[ "MAGE" ] = "DAMAGER",
	[ "PALADIN" ] = { "HEALER", "TANK", "DAMAGER", },
	[ "PRIEST" ] = { "HEALER", "HEALER", "DAMAGER", },
	[ "ROGUE" ] = "DAMAGER",
	[ "SHAMAN" ] = { "DAMAGER", "DAMAGER", "HEALER", },
	[ "WARLOCK" ] = "DAMAGER",
	[ "WARRIOR" ] = { "DAMAGER", "DAMAGER", "TANK", },
}

-- Determines the best role for a particular class and talent spec.
-- @param class a class name.
-- @param spec the primary talent tree index.
-- @return one of "DAMAGER, "HEALER", or "TANK"
local function _GetSpecRole( class, spec )
	local roles = _ROLES[ class ]
	if type( roles ) == "table" then
		local role = roles[ spec ]
		if type( role ) == "function" then
			return role( )
		end
		return role
	end
	return roles
end

-- Determines the best role for the player, based on the player class 
-- and current talent spec.
local function _PlayerGetSpecRole( )
	local _, playerClass = UnitClass( "player" )
	local playerSpec = GetPrimaryTalentTree( false, false )
	return _GetSpecRole( playerClass, playerSpec )
end

-- Sets the player role based on the class and active spec. If the role 
-- is already set to the desired role, no change is made.
local function _PlayerSetSpecRole( role )
	role = role or _PlayerGetSpecRole( )
	if role == _ROLE_NONE then
		return nil
	end
	
	local current = UnitGroupRolesAssigned( "player" )
	if role ~= current then
		UnitSetRole( "player", role )
	end
	
	return role
end

--- Fires when the player switches talent builds.
-- Updates the cached role.
function addon:ACTIVE_TALENT_GROUP_CHANGED( event )
	self.state.role = _PlayerSetSpecRole( )
end

--- Fires when group membership information changes.
-- This is used to check when the player joins some sort of group, or 
-- a role change is made for any group member. If the player's role is 
-- changed to a role different than the cached role, the 
-- role is reset to the cached role. If the player is no 
-- longer in a group, the cached role is cleared.
function addon:PARTY_MEMBERS_CHANGED( event )
	if not _IsPlayerInGroup( ) then
		self.state.role = nil
		return
	end
	
	self.state.role = _PlayerSetSpecRole( self.state.role )
end

--- Fires when a group member's role is changed.
-- If the player role is changed by the player, this will override the 
-- cached role. If any other unit attempts to change the 
-- player's role, it will be reset to the cached role.
-- @param target the unit changing to a new role.
-- @param changer the unit changing the role.
-- @param oldRole the old role.
-- @param newRole the new role.
function addon:ROLE_CHANGED_INFORM( event, target, changer, oldRole, newRole )
	local name, _ = UnitName( "player" )
	if ( target == name ) and ( changer == name ) then
		self.state.role = newRole
	end
end

--- Fires when a role check is started for a group.
-- This suppresses the role popup, which is unnecessary for users of 
-- this addon.
function addon:ROLE_POLL_BEGIN( event )
	StaticPopupSpecial_Hide( RolePollPopup )
end

-- Register the event handler and initial events.
addon:SetScript( "OnEvent", addon.OnEvent )
addon:RegisterEvent( "PLAYER_LOGIN" )
